// Author:   CodePug (www.codepug.com)
// Purpose:  Control loading, painting, & sprite animation
// Filename: xsprite.h
// Compiler: Win32 Lcc
// Note: If you use this code, you must show credit to the original
//       author, CodePug .  Any exceptions to this
//	     will be considered illegal.
//
// Todo: sprite magnification
// Last updated: Feb 27 2003
#ifndef _XSPRITE_H				// If we haven't included this file
#define _XSPRITE_H				// Set a flag saying we included it

#include "xgraphics.h"

typedef struct _SPRITE {
	// Core sprite variables
	HBITMAP image;
	POINT pos,
	      course;
	int width,
	    height,
		speed,
		hp;
	BOOL isActive,
		 isFlipable;
	long nextSpritePtr;
	char spriteName[256],
	     groupName[256];

	// Transparent image variables
	HBITMAP imageMask;
	BOOL isTransparent;

	// Flipped image variables
	HBITMAP imageFlip,
	        imageFlipMask;
	BOOL isFlipped;

	// Animation variables
	BOOL isAnimation,
	     loopAnimationHideWhenDone,
		 loopAnimationFreezeWhenDone,
		 autoUpdateFrame;
	int  currentImagePosition,
	     maxImagePosition,
		 frameSpeed;

	// Animation slower then fps variables
	int repeatFrameCount;
	int maxRepeatPosition;
	int repeatPosition;

} SPRITE;

///////////// Global Variables /////////////////
//
SPRITE *spriteStack[25];
int currentSpriteStack=0;
int maxSpriteStack = 0;
COLORREF transparentColor = RGB(255,0,255);

//////////// Method Prototypes ////////////////
//
void addSprite(SPRITE *sp);
void paintSprite(HDC h, HDC temp);
SPRITE createSprite(char *imageName, BOOL isThisTransparent, BOOL isThisFlipable);
SPRITE createAnimatedSprite(char *imageName, BOOL isThisTransparent, BOOL isThisFlipable, int animationNumberOfFrames);
void removeSprite(SPRITE *sp);
void setDimensions(SPRITE *sp, int width, int height);
RECT getDimensions(SPRITE *sp);
POINT getPositionCenter(SPRITE *sp);
POINT getPostion(SPRITE *sp);
void setPosition(SPRITE *sp, int x, int y);
void setPositionCenter(SPRITE *sp, int x, int y);
void setPositionFromSprite(SPRITE *src,SPRITE *dest);
BOOL checkCollision(SPRITE *sp,int x, int y);
void setCourse(SPRITE *sp, int x, int y,int speed);
BOOL followCourse(SPRITE *sp);
void moveSprite(SPRITE *sp, int changeInX, int changeInY);
void deactivateGroup(char *groupName);
void setPositionGroup(char *groupName,int x, int y);
void moveSpriteGroup(char *groupName, int changeInX, int changeInY);
void flipGroup(char *groupName,BOOL flip);
void deleteAllSpriteObjects();

void moveSprite(SPRITE *sp, int changeInX, int changeInY){
	sp->pos.x = sp->pos.x + changeInX;
	sp->pos.y = sp->pos.y + changeInY;
}

RECT getDimensions(SPRITE *sp){
	RECT r;
	r.left   = 0;
	r.top    = 0;
	r.right  = sp->width;
	r.bottom = sp->height;
	return r;
}

void setDimensions(SPRITE *sp, int width, int height){
	sp->width  = width;
	sp->height = height;
}

POINT getPostion(SPRITE *sp){
	return sp->pos;
}

POINT getPositionCenter(SPRITE *sp){
	POINT p;
	p.x = sp->pos.x + (sp->width/2);
	p.y = sp->pos.y + (sp->height/2);
	return p;
}

void setPosition(SPRITE *sp, int x, int y){
	sp->pos.x = x;
	sp->pos.y = y;
}

void setPositionCenter(SPRITE *sp, int x, int y){
	sp->pos.x = x - (sp->width/2);
	sp->pos.y = y - (sp->height/2);
}

void setPositionFromSprite(SPRITE *src,SPRITE *dest){
	POINT p;
	p = getPositionCenter(src);
	setPositionCenter(dest,p.x,p.y);
}

void setCourse(SPRITE *sp, int x, int y,int speed){
	sp->course.x = x;
	sp->course.y = y;
	sp->speed    = speed;
}

BOOL followCourse(SPRITE *sp){
	POINT p;
	BOOL hasMoved = FALSE;

	p = getPositionCenter(sp);
	// Uncomment to use top left corner instead of center
	//p.x = sp->pos.x;
	//p.y = sp->pos.y;

	if (sp->course.x > p.x+sp->speed/2){
		sp->pos.x+=sp->speed;
		hasMoved = TRUE;
	}
	if (sp->course.x < p.x-sp->speed/2){
		sp->pos.x-=sp->speed;
		hasMoved = TRUE;
	}
	if (sp->course.y < p.y+sp->speed/2){
		sp->pos.y-=sp->speed;
		hasMoved = TRUE;
	}
	if (sp->course.y > p.y-sp->speed/2){
		sp->pos.y+=sp->speed;
		hasMoved = TRUE;
	}
	return hasMoved;
}


BOOL checkCollision(SPRITE *sp,int x, int y){
	if (sp->pos.x < x && (sp->pos.x+sp->width) > x)
		if (sp->pos.y < y && (sp->pos.y+sp->height) > y)
			return TRUE;

	return FALSE;
}


SPRITE createSprite(char *imageName, BOOL isThisTransparent, BOOL isThisFlipable){
	SPRITE sp;
	BITMAP bm;
	memset(&sp,0,sizeof(SPRITE));

	sp.image    = LoadABitmap(imageName);
	if (sp.image == NULL)
		sp.image = LoadBitmap(hInst,MAKEINTRESOURCE(atoi(imageName)));

	sp.isActive = TRUE;
	sp.pos.x = 0;
	sp.pos.y = 0;
	sp.speed = 1;
	sp.course.x = 0;
	sp.course.y = 0;
	sp.hp    = 2;
	strcpy(sp.groupName,"sprite");

	// Transparency settings
	sp.isTransparent = isThisTransparent;
	if (isThisTransparent)
		sp.imageMask = CreateBitmapMask(sp.image, transparentColor);

	// Total Image size
	GetObject(sp.image, sizeof(bm), &bm );
	sp.width  = bm.bmWidth;
    sp.height = bm.bmHeight;

	// Animation settings
	sp.isAnimation     = FALSE;
	sp.autoUpdateFrame = TRUE;
	sp.loopAnimationHideWhenDone   = FALSE;
	sp.loopAnimationFreezeWhenDone = FALSE;
	sp.currentImagePosition = 0;
	sp.maxImagePosition     = 0;
	sp.frameSpeed           = 7;

	// Animation slower then fps variables
	sp.maxRepeatPosition    = 20;
	sp.repeatFrameCount = 0;


	// Flip image settings
	sp.isFlipped = FALSE;
	sp.isFlipable = isThisFlipable;
	if (isThisFlipable){
		sp.imageFlip     = LoadAndFlipBitmap(sp.image,hwndMain);
		sp.imageFlipMask = CreateBitmapMask(sp.imageFlip, transparentColor);
	}

	return sp;
} // end createSprite

SPRITE createAnimatedSprite(char *imageName, BOOL isThisTransparent, BOOL isThisFlipable, int animationNumberOfFrames){
	SPRITE sp;

	memset(&sp,0,sizeof(SPRITE));
	sp = createSprite(imageName,isThisTransparent,isThisFlipable);

	// Animation specific settings
	sp.isAnimation = TRUE;
	sp.maxImagePosition = animationNumberOfFrames;

	// Individual image size
	sp.width  = sp.width / animationNumberOfFrames;

	return sp;
} // end createSprite


void addSprite(SPRITE *sp){
	spriteStack[maxSpriteStack] = sp;
	maxSpriteStack++;
}

void deleteAllSpriteObjects(){
	int i;
	for (i=maxSpriteStack-1; i>=0;i--){
		DeleteObject(spriteStack[i]->image);
		DeleteObject(spriteStack[i]->imageMask);
	}
}

void removeSprite(SPRITE *sp){
	maxSpriteStack--;
	DeleteObject(sp->image);
	DeleteObject(sp->imageMask);
}

void flipGroup(char *groupName,BOOL flip){
	int i;

	for (i=maxSpriteStack-1; i>=0;i--)
		if (!strcmp(spriteStack[i]->groupName,groupName))
			if (spriteStack[i]->isFlipable)
				spriteStack[i]->isFlipped = flip;
}

void deactivateGroup(char *groupName){
	int i;

	for (i=maxSpriteStack-1; i>=0;i--)
		if (!strcmp(spriteStack[i]->groupName,groupName))
			spriteStack[i]->isActive = FALSE;
}

void setPositionGroup(char *groupName,int x, int y){
	int i;

	for (i=maxSpriteStack-1; i>=0;i--)
		if (!strcmp(spriteStack[i]->groupName,groupName)){
			spriteStack[i]->pos.x = x;
			spriteStack[i]->pos.y = y;
		}
}

void moveSpriteGroup(char *groupName, int changeInX, int changeInY){
	int i;

	for (i=maxSpriteStack-1; i>=0;i--)
		if (!strcmp(spriteStack[i]->groupName,groupName)){
			spriteStack[i]->pos.x = spriteStack[i]->pos.x + changeInX;
			spriteStack[i]->pos.y = spriteStack[i]->pos.y + changeInY;
		}
}

void paintSprite(HDC h, HDC temp){
	int i,
		frameXPosition;

	// Paint in reverse
	for (i=maxSpriteStack-1; i>=0;i--){
		// Quick hitpoint code
		if (spriteStack[i]->hp == 0)
			spriteStack[i]->isActive = FALSE;

		// Check to see if we should show it
		if (spriteStack[i]->isActive){

			// Sprite animation & non-animation update actions
			if (spriteStack[i]->isAnimation){
				frameXPosition = (spriteStack[i]->currentImagePosition)*(spriteStack[i]->width);

				// Animation update actions
				if (spriteStack[i]->currentImagePosition < spriteStack[i]->maxImagePosition-1){
					if (spriteStack[i]->autoUpdateFrame)

						if ( spriteStack[i]->repeatFrameCount > spriteStack[i]->maxRepeatPosition){
							spriteStack[i]->repeatFrameCount=0;
							spriteStack[i]->currentImagePosition++;

						}else
						    spriteStack[i]->repeatFrameCount++;
				} else {
// Todo: cleanup start
					if (!spriteStack[i]->loopAnimationFreezeWhenDone){

							if ( spriteStack[i]->repeatFrameCount > spriteStack[i]->maxRepeatPosition){
							spriteStack[i]->repeatFrameCount=0;
							spriteStack[i]->currentImagePosition=0;
							}else
						    spriteStack[i]->repeatFrameCount++;

// End todo: cleanup
					}else
						spriteStack[i]->autoUpdateFrame = FALSE;

					if (spriteStack[i]->loopAnimationHideWhenDone){
						spriteStack[i]->isActive = FALSE;
						spriteStack[i]->currentImagePosition = 0;
					}
				}
			} else // Now do update for non animations
				frameXPosition = 0;

		// paint all active graphics both anim and regular
		if (!spriteStack[i]->isFlipped){
			if (spriteStack[i]->isTransparent){
				//SelectObject(temp, spriteStack[i]->image);
			   	//BitBlt(h2, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCINVERT);
		//temp = CreateCompatibleDC(h);
				SelectObject(temp, spriteStack[i]->imageMask);
			   	BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCAND);

			SelectObject(temp, spriteStack[i]->image);
   			BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCPAINT);
//BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,h2,frameXPosition,0, SRCCOPY);

			} else {
				SelectObject(temp, spriteStack[i]->image);
	   			BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCCOPY);
//BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCINVERT);

			}
		// Then is flipped
		} else {
				if (spriteStack[i]->isTransparent){
				SelectObject(temp, spriteStack[i]->imageFlipMask);
			   	BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCAND);

				SelectObject(temp, spriteStack[i]->imageFlip);
	   			BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCPAINT);
			} else {
				SelectObject(temp, spriteStack[i]->imageFlip);
	   			BitBlt(h, spriteStack[i]->pos.x,spriteStack[i]->pos.y,spriteStack[i]->width,spriteStack[i]->height,temp,frameXPosition,0, SRCCOPY);
			}
		} // endif !isFlipped

		} // end active
	} // end for loop
} // end paintSprite

#endif




